@@ -149,34 +149,6 @@ typedef struct bits_image bits_image_t;
 typedef struct circle circle_t;
 typedef struct point point_t;
 
-typedef void (* CompositeFunc) (pixman_op_t,
-				pixman_image_t *, pixman_image_t *, pixman_image_t *,
-				int16_t, int16_t, int16_t, int16_t, int16_t, int16_t,
-				uint16_t, uint16_t);
-
-
-/* These "formats" both have depth 0, so they
- * will never clash with any real ones
- */
-#define PIXMAN_null		PIXMAN_FORMAT(0,0,0,0,0,0)
-#define PIXMAN_solid		PIXMAN_FORMAT(0,1,0,0,0,0)
-
-#define NEED_COMPONENT_ALPHA		(1 << 0)
-#define NEED_PIXBUF			(1 << 1)
-#define NEED_SOLID_MASK		        (1 << 2)
-
-typedef struct
-{
-    pixman_op_t			op;
-    pixman_format_code_t	src_format;
-    pixman_format_code_t	mask_format;
-    pixman_format_code_t	dest_format;
-    CompositeFunc		func;
-    uint32_t			flags;
-} FastPathInfo;
-
-extern const FastPathInfo *const c_fast_paths;
-
 /* FIXME - the types and structures below should be give proper names
  */
 
@@ -212,19 +184,6 @@ typedef struct _FbComposeData {
     uint16_t	 height;
 } FbComposeData;
 
-typedef struct _FbComposeFunctions32 {
-    CombineFunc32 *combineU;
-    CombineFunc32 *combineC;
-} FbComposeFunctions32;
-
-typedef struct _FbComposeFunctions64 {
-    CombineFunc64 *combineU;
-    CombineFunc64 *combineC;
-} FbComposeFunctions64;
-
-extern FbComposeFunctions32 pixman_composeFunctions;
-extern FbComposeFunctions64 pixman_composeFunctions64;
-
 void pixman_composite_rect_general_accessors (const FbComposeData *data,
                                               void *src_buffer,
                                               void *mask_buffer,
@@ -255,8 +214,6 @@ void _pixman_image_get_scanline_64_generic (pixman_image_t * pict, int x, int y,
 					    uint64_t *buffer, uint64_t *mask, uint32_t maskBits);
 void fbFetchTransformed(bits_image_t *, int x, int y, int width,
                         uint32_t *buffer, uint32_t *mask, uint32_t maskBits);
-void fbStoreExternalAlpha(bits_image_t *, int x, int y, int width,
-                          uint32_t *buffer);
 void fbFetchExternalAlpha(bits_image_t *, int x, int y, int width,
                           uint32_t *buffer, uint32_t *mask, uint32_t maskBits);
 
@@ -269,12 +226,6 @@ void fbFetchExternalAlpha_accessors(bits_image_t *, int x, int y, int width,
                                     uint32_t *buffer, uint32_t *mask,
                                     uint32_t maskBits);
 
-void fbStoreExternalAlpha64(bits_image_t *, int x, int y, int width,
-                            uint64_t *buffer);
-
-void fbStoreExternalAlpha64_accessors(bits_image_t *, int x, int y, int width,
-                                      uint64_t *buffer);
-
 /* end */
 
 typedef enum
@@ -295,7 +246,7 @@ typedef enum
     SOURCE_IMAGE_CLASS_VERTICAL,
 } source_pict_class_t;
 
-typedef void (*scanStoreProc)(pixman_image_t *, int, int, int, uint32_t *);
+typedef void (*scanStoreProc)(bits_image_t *img, int x, int y, int width, uint32_t *buffer);
 typedef void (*scanFetchProc)(pixman_image_t *, int, int, int, uint32_t *,
 			      uint32_t *, uint32_t);
 
@@ -316,9 +267,15 @@ void
 _pixman_image_get_scanline_64 (pixman_image_t *image, int x, int y, int width,
 			       uint32_t *buffer, uint32_t *unused, uint32_t unused2);
 
-scanStoreProc
-_pixman_image_get_storer (pixman_image_t *image,
-			  int             wide);
+void
+_pixman_image_store_scanline_32 (bits_image_t *image, int x, int y, int width,
+				 uint32_t *buffer);
+/* Even thought the type of buffer is uint32_t *, the function actually expects
+ * a uint64_t *buffer.
+ */
+void
+_pixman_image_store_scanline_64 (bits_image_t *image, int x, int y, int width,
+				 uint32_t *buffer);
 
 pixman_image_t *
 _pixman_image_allocate (void);
@@ -431,6 +388,9 @@ struct bits_image
     uint32_t *			bits;
     uint32_t *			free_me;
     int				rowstride; /* in number of uint32_t's */
+
+    scanStoreProc		store_scanline_32;
+    scanStoreProc		store_scanline_64;
 };
 
 union pixman_image
@@ -674,9 +634,9 @@ _pixman_gradient_walker_pixel (GradientWalker       *walker,
 
 #define READ(img, ptr)		(*(ptr))
 #define WRITE(img, ptr, val)	(*(ptr) = (val))
-#define MEMCPY_WRAPPED(img, dst, src, size)					\
+#define MEMCPY_WRAPPED(img, dst, src, size)				\
     memcpy(dst, src, size)
-#define MEMSET_WRAPPED(img, dst, val, size)					\
+#define MEMSET_WRAPPED(img, dst, val, size)				\
     memset(dst, val, size)
 
 #endif
@@ -701,7 +661,7 @@ _pixman_gradient_walker_pixel (GradientWalker       *walker,
 		(res) = READ(img, (uint32_t *)bits__);			\
 		break;							\
 	    case 24:							\
-		(res) = Fetch24(img, (uint8_t *) bits__);			\
+		(res) = Fetch24(img, (uint8_t *) bits__);		\
 		break;							\
 	    case 16:							\
 		(res) = READ(img, (uint16_t *) bits__);			\
@@ -893,4 +853,229 @@ void pixman_timer_register (PixmanTimer *timer);
 
 #endif /* PIXMAN_TIMING */
 
+typedef struct pixman_implementation_t pixman_implementation_t;
+
+typedef void (* pixman_combine_32_func_t) (pixman_implementation_t *	imp,
+					   pixman_op_t			op,
+					   uint32_t *			dest,
+					   const uint32_t *		src,
+					   const uint32_t *		mask,
+					   int				width);
+
+typedef void (* pixman_combine_64_func_t) (pixman_implementation_t *	imp,
+					   pixman_op_t			op,
+					   uint64_t *			dest,
+					   const uint64_t *		src,
+					   const uint64_t *		mask,
+					   int				width);
+
+typedef void (* pixman_composite_func_t)  (pixman_implementation_t *	imp,
+					   pixman_op_t			op,
+					   pixman_image_t *		src,
+					   pixman_image_t *		mask,
+					   pixman_image_t *		dest,
+					   int32_t			src_x,
+					   int32_t			src_y,
+					   int32_t			mask_x,
+					   int32_t			mask_y,
+					   int32_t			dest_x,
+					   int32_t			dest_y,
+					   int32_t			width,
+					   int32_t			height);
+typedef pixman_bool_t (* pixman_blt_func_t) (pixman_implementation_t *	imp,
+					     uint32_t *			src_bits,
+					     uint32_t *			dst_bits,
+					     int			src_stride,
+					     int			dst_stride,
+					     int			src_bpp,
+					     int			dst_bpp,
+					     int			src_x,
+					     int			src_y,
+					     int			dst_x,
+					     int			dst_y,
+					     int			width,
+					     int			height);
+typedef pixman_bool_t (* pixman_fill_func_t) (pixman_implementation_t *imp,
+					      uint32_t *bits,
+					      int stride,
+					      int bpp,
+					      int x,
+					      int y,
+					      int width,
+					      int height,
+					      uint32_t xor);
+
+void
+_pixman_walk_composite_region (pixman_implementation_t *imp,
+			      pixman_op_t op,
+			      pixman_image_t * pSrc,
+			      pixman_image_t * pMask,
+			      pixman_image_t * pDst,
+			      int16_t xSrc,
+			      int16_t ySrc,
+			      int16_t xMask,
+			      int16_t yMask,
+			      int16_t xDst,
+			      int16_t yDst,
+			      uint16_t width,
+			      uint16_t height,
+			      pixman_bool_t srcRepeat,
+			      pixman_bool_t maskRepeat,
+			       pixman_composite_func_t compositeRect);
+
+void _pixman_setup_combiner_functions_32 (pixman_implementation_t *imp);
+void _pixman_setup_combiner_functions_64 (pixman_implementation_t *imp);
+
+/* These "formats" both have depth 0, so they
+ * will never clash with any real ones
+ */
+#define PIXMAN_null		PIXMAN_FORMAT(0,0,0,0,0,0)
+#define PIXMAN_solid		PIXMAN_FORMAT(0,1,0,0,0,0)
+
+#define NEED_COMPONENT_ALPHA		(1 << 0)
+#define NEED_PIXBUF			(1 << 1)
+#define NEED_SOLID_MASK		        (1 << 2)
+
+typedef struct
+{
+    pixman_op_t			op;
+    pixman_format_code_t	src_format;
+    pixman_format_code_t	mask_format;
+    pixman_format_code_t	dest_format;
+    pixman_composite_func_t	func;
+    uint32_t			flags;
+} FastPathInfo;
+
+struct pixman_implementation_t
+{
+    pixman_implementation_t *	toplevel;
+    pixman_implementation_t *	delegate;
+
+    pixman_composite_func_t	composite;
+    pixman_blt_func_t		blt;
+    pixman_fill_func_t		fill;
+    
+    pixman_combine_32_func_t	combine_32[PIXMAN_OP_LAST];
+    pixman_combine_32_func_t	combine_32_ca[PIXMAN_OP_LAST];
+    pixman_combine_64_func_t	combine_64[PIXMAN_OP_LAST];
+    pixman_combine_64_func_t	combine_64_ca[PIXMAN_OP_LAST];
+};
+
+pixman_implementation_t *
+_pixman_implementation_create (pixman_implementation_t *toplevel,
+			       pixman_implementation_t *delegate);
+
+void
+_pixman_implementation_combine_32 (pixman_implementation_t *	imp,
+				   pixman_op_t			op,
+				   uint32_t *			dest,
+				   const uint32_t *		src,
+				   const uint32_t *		mask,
+				   int				width);
+void
+_pixman_implementation_combine_64 (pixman_implementation_t *	imp,
+				   pixman_op_t			op,
+				   uint64_t *			dest,
+				   const uint64_t *		src,
+				   const uint64_t *		mask,
+				   int				width);
+void
+_pixman_implementation_combine_32_ca (pixman_implementation_t *	imp,
+				      pixman_op_t		op,
+				      uint32_t *		dest,
+				      const uint32_t *		src,
+				      const uint32_t *		mask,
+				      int			width);
+void
+_pixman_implementation_combine_64_ca (pixman_implementation_t *	imp,
+				      pixman_op_t		op,
+				      uint64_t *		dest,
+				      const uint64_t *		src,
+				      const uint64_t *		mask,
+				      int			width);
+void
+_pixman_implementation_composite (pixman_implementation_t *	imp,
+				  pixman_op_t			op,
+				  pixman_image_t *		src,
+				  pixman_image_t *		mask,
+				  pixman_image_t *		dest,
+				  int32_t			src_x,
+				  int32_t			src_y,
+				  int32_t			mask_x,
+				  int32_t			mask_y,
+				  int32_t			dest_x,
+				  int32_t			dest_y,
+				  int32_t			width,
+				  int32_t			height);
+
+pixman_bool_t
+_pixman_implementation_blt (pixman_implementation_t *	imp,
+			    uint32_t *			src_bits,
+			    uint32_t *			dst_bits,
+			    int				src_stride,
+			    int				dst_stride,
+			    int				src_bpp,
+			    int				dst_bpp,
+			    int				src_x,
+			    int				src_y,
+			    int				dst_x,
+			    int				dst_y,
+			    int				width,
+			    int				height);
+pixman_bool_t
+_pixman_implementation_fill (pixman_implementation_t *   imp,
+			     uint32_t *bits,
+			     int stride,
+			     int bpp,
+			     int x,
+			     int y,
+			     int width,
+			     int height,
+			     uint32_t xor);
+    
+/* Specific implementations */
+pixman_implementation_t *
+_pixman_implementation_create_general (pixman_implementation_t *toplevel);
+pixman_implementation_t *
+_pixman_implementation_create_fast_path (pixman_implementation_t *toplevel);
+#ifdef USE_MMX
+pixman_implementation_t *
+_pixman_implementation_create_mmx (pixman_implementation_t *toplevel);
+#endif
+#ifdef USE_SSE2
+pixman_implementation_t *
+_pixman_implementation_create_sse2 (pixman_implementation_t *toplevel);
+#endif
+#ifdef USE_ARM_SIMD
+pixman_implementation_t *
+_pixman_implementation_create_arm_simd (pixman_implementation_t *toplevel);
+#endif
+#ifdef USE_ARM_NEON
+pixman_implementation_t *
+_pixman_implementation_create_arm_neon (pixman_implementation_t *toplevel);
+#endif
+#ifdef USE_VMX
+pixman_implementation_t *
+_pixman_implementation_create_vmx (pixman_implementation_t *toplevel);
+#endif
+
+pixman_bool_t
+_pixman_run_fast_path (const FastPathInfo *paths,
+		       pixman_implementation_t *imp,
+		       pixman_op_t op,
+		       pixman_image_t *src,
+		       pixman_image_t *mask,
+		       pixman_image_t *dest,
+		       int32_t src_x,
+		       int32_t src_y,
+		       int32_t mask_x,
+		       int32_t mask_y,
+		       int32_t dest_x,
+		       int32_t dest_y,
+		       int32_t width,
+		       int32_t height);
+    
+pixman_implementation_t *
+_pixman_choose_implementation (void);
+
 #endif /* PIXMAN_PRIVATE_H */
